`define NUM_CORE 2

module oc_warp_tmu_top(handsome_clk, handsome_din_top, handsome_dout_top, sys_clk, sys_rst);
	input handsome_clk;
	input handsome_din_top;
	output handsome_dout_top;
	input sys_clk;
	input sys_rst;

localparam input_width = 146;
localparam output_width = 196;
localparam reducer_log4 = 4;
localparam reducer_log4_top = 4;

wire [`NUM_CORE-1:0] csr_we;
wire [`NUM_CORE-1:0] wbm_ack_i;
wire [`NUM_CORE-1:0] fmlr_ack;
wire [`NUM_CORE-1:0] fmlw_ack;
wire [`NUM_CORE-1:0] irq;
wire [`NUM_CORE-1:0] wbm_cyc_o;
wire [`NUM_CORE-1:0] wbm_stb_o;
wire [`NUM_CORE-1:0] fmlr_stb;
wire [`NUM_CORE-1:0] fmlw_stb;
wire [13:0] csr_a [`NUM_CORE-1:0];
wire [31:0] csr_di [`NUM_CORE-1:0];
wire [31:0] wbm_dat_i [`NUM_CORE-1:0];
wire [63:0] fmlr_di [`NUM_CORE-1:0];
wire [31:0] csr_do [`NUM_CORE-1:0];
wire [31:0] wbm_adr_o [`NUM_CORE-1:0];
wire [2:0] wbm_cti_o [`NUM_CORE-1:0];
wire [25:0] fmlr_adr [`NUM_CORE-1:0];
wire [25:0] fmlw_adr [`NUM_CORE-1:0];
wire [7:0] fmlw_sel [`NUM_CORE-1:0];
wire [63:0] fmlw_do [`NUM_CORE-1:0];
wire [`NUM_CORE-1:0] handsome_din;
wire [`NUM_CORE-1:0] handsome_dout;

handsome_lfsr #(`NUM_CORE) input_lfsr_top (
			.clock(handsome_clk),
			.input_bit(handsome_din_top),
			.output_vector(handsome_din));

generate
begin:G1
	genvar i;
	for (i = 0; i < `NUM_CORE; i = i+1)
	begin :stamp_inst

		handsome_lfsr #(input_width) input_lfsr (
			.clock(handsome_clk),
			.input_bit(handsome_din[i]),
			.output_vector({csr_we[i],wbm_ack_i[i],fmlr_ack[i],fmlw_ack[i],csr_a[i],csr_di[i],wbm_dat_i[i],fmlr_di[i]}));

		oc_warp_tmu oc_stamp(
			.sys_clk(sys_clk),
			.sys_rst(sys_rst),
			.csr_we(csr_we[i]),
			.wbm_ack_i(wbm_ack_i[i]),
			.fmlr_ack(fmlr_ack[i]),
			.fmlw_ack(fmlw_ack[i]),
			.csr_a(csr_a[i]),
			.csr_di(csr_di[i]),
			.wbm_dat_i(wbm_dat_i[i]),
			.fmlr_di(fmlr_di[i]),
			.irq(irq[i]),
			.wbm_cyc_o(wbm_cyc_o[i]),
			.wbm_stb_o(wbm_stb_o[i]),
			.fmlr_stb(fmlr_stb[i]),
			.fmlw_stb(fmlw_stb[i]),
			.csr_do(csr_do[i]),
			.wbm_adr_o(wbm_adr_o[i]),
			.wbm_cti_o(wbm_cti_o[i]),
			.fmlr_adr(fmlr_adr[i]),
			.fmlw_adr(fmlw_adr[i]),
			.fmlw_sel(fmlw_sel[i]),
			.fmlw_do(fmlw_do[i]));

		handsome_reducer #(reducer_log4) output_reducer (
			.clock(handsome_clk),
			.input_vector({irq[i],wbm_cyc_o[i],wbm_stb_o[i],fmlr_stb[i],fmlw_stb[i],csr_do[i],wbm_adr_o[i],wbm_cti_o[i],fmlr_adr[i],fmlw_adr[i],fmlw_sel[i],fmlw_do[i]}),
			.output_bit(handsome_dout[i]));

	end

handsome_reducer #(reducer_log4_top) output_reducer_top (
	.clock(handsome_clk),
	.input_vector(handsome_dout[`NUM_CORE-1:0]),
	.output_bit(handsome_dout_top));

end
endgenerate
endmodule
